@@ -17,7 +17,12 @@ from mltrace.client import (
     set_db_uri,
     get_db_uri,
     get_git_hash,
+    get_git_tags,
     set_address,
+    add_notes_to_component_run,
+    flag_output_id,
+    unflag_output_id,
+    review_flagged_outputs,
 )
 
 __all__ = [
@@ -39,5 +44,10 @@ __all__ = [
     "set_db_uri",
     "get_db_uri",
     "get_git_hash",
+    "get_git_tags",
     "set_address",
+    "add_notes_to_component_run",
+    "flag_output_id",
+    "unflag_output_id",
+    "review_flagged_outputs",
 ]
