@@ -26,10 +26,11 @@ if _db_uri is None:
         _db_uri = _set_address_helper(_db_uri, os.environ.get("DB_SERVER"))
     else:
         logging.warning(
-            f"Please set DB_URI or DB_SERVER as an environment variable. Otherwise, DB_URI is set to {_db_uri}."
+            f"Please set DB_URI or DB_SERVER as an environment variable. \
+            Otherwise, DB_URI is set to {_db_uri}."
         )
 
-# ----------------------- Database management functions ---------------------- #
+# --------------------- Database management functions ------------------- #
 
 
 def set_db_uri(uri: str):
@@ -52,7 +53,7 @@ def clean_db():
     store = Store(_db_uri, delete_first=True)
 
 
-# ---------------------------- Creation functions ---------------------------- #
+# ----------------------- Creation functions ---------------------------- #
 
 
 def create_component(
@@ -69,7 +70,11 @@ def tag_component(component_name: str, tags: typing.List[str]):
     store.add_tags_to_component(component_name, tags)
 
 
-def log_component_run(component_run: ComponentRun, set_dependencies_from_inputs=True):
+def log_component_run(
+    component_run: ComponentRun,
+    set_dependencies_from_inputs=True,
+    staleness_threshold: int = (60 * 60 * 24 * 30),
+):
     """Takes client-facing ComponentRun object and logs it to the DB."""
     store = Store(_db_uri)
 
@@ -82,12 +87,20 @@ def log_component_run(component_run: ComponentRun, set_dependencies_from_inputs=
 
     # Add relevant attributes
     if component_run_dict["start_timestamp"]:
-        component_run_sql.set_start_timestamp(component_run_dict["start_timestamp"])
+        component_run_sql.set_start_timestamp(
+            component_run_dict["start_timestamp"]
+        )
 
     if component_run_dict["end_timestamp"]:
-        component_run_sql.set_end_timestamp(component_run_dict["end_timestamp"])
+        component_run_sql.set_end_timestamp(
+            component_run_dict["end_timestamp"]
+        )
+
+    if component_run_dict["notes"]:
+        component_run_sql.add_notes(component_run_dict["notes"])
 
     component_run_sql.set_git_hash(component_run_dict["git_hash"])
+    component_run_sql.set_git_tags(component_run_dict["git_tags"])
     component_run_sql.set_code_snapshot(component_run_dict["code_snapshot"])
 
     # Add I/O
@@ -113,11 +126,14 @@ def log_component_run(component_run: ComponentRun, set_dependencies_from_inputs=
         cr = store.get_history(dependency, 1)[0]
         component_run_sql.set_upstream(cr)
 
-    store.commit_component_run(component_run_sql)
+    store.commit_component_run(
+        component_run_sql, staleness_threshold=staleness_threshold
+    )
 
 
 def create_random_ids(num_outputs=1) -> typing.List[str]:
-    """Returns a list of num_outputs ids that a client can use to tag outputs."""
+    """Returns a list of num_outputs ids
+    that a client can use to tag outputs."""
 
     return [str(uuid.uuid4()) for _ in range(num_outputs)]
 
@@ -133,6 +149,7 @@ def register(
     input_vars: typing.List[str] = [],
     output_vars: typing.List[str] = [],
     endpoint: bool = False,
+    staleness_threshold: int = (60 * 60 * 24 * 30),
 ):
     def actual_decorator(func):
         @functools.wraps(func)
@@ -143,7 +160,9 @@ def register(
 
             # Construct component run object
             store = Store(_db_uri)
-            component_run = store.initialize_empty_component_run(component_name)
+            component_run = store.initialize_empty_component_run(
+                component_name
+            )
             component_run.set_start_timestamp()
 
             # Define trace helper
@@ -158,10 +177,12 @@ def register(
                 # Add input_vars and output_vars as pointers
                 for var in input_vars:
                     if var not in local_vars:
-                        logging.debug(f"Variable {var} not in current stack frame.")
+                        logging.debug(
+                            f"Variable {var} not in current stack frame."
+                        )
                         continue
                     val = local_vars[var]
-                    if val == None:
+                    if val is None:
                         logging.debug(f"Variable {var} has value {val}.")
                         continue
                     if isinstance(val, list):
@@ -170,21 +191,29 @@ def register(
                         input_pointers.append(store.get_io_pointer(str(val)))
                 for var in output_vars:
                     if var not in local_vars:
-                        logging.debug(f"Variable {var} not in current stack frame.")
+                        logging.debug(
+                            f"Variable {var} not in current stack frame."
+                        )
                         continue
                     val = local_vars[var]
-                    if val == None:
+                    if val is None:
                         logging.debug(f"Variable {var} has value {val}.")
                         continue
                     if isinstance(val, list):
                         output_pointers += (
-                            store.get_io_pointers(val, PointerTypeEnum.ENDPOINT)
+                            store.get_io_pointers(
+                                val, PointerTypeEnum.ENDPOINT
+                            )
                             if endpoint
                             else store.get_io_pointers(val)
                         )
                     else:
                         output_pointers += (
-                            [store.get_io_pointer(str(val), PointerTypeEnum.ENDPOINT)]
+                            [
+                                store.get_io_pointer(
+                                    str(val), PointerTypeEnum.ENDPOINT
+                                )
+                            ]
                             if endpoint
                             else [store.get_io_pointer(str(val))]
                         )
@@ -212,7 +241,10 @@ def register(
             component_run.set_end_timestamp()
             input_pointers = [store.get_io_pointer(inp) for inp in inputs]
             output_pointers = (
-                [store.get_io_pointer(out, PointerTypeEnum.ENDPOINT) for out in outputs]
+                [
+                    store.get_io_pointer(out, PointerTypeEnum.ENDPOINT)
+                    for out in outputs
+                ]
                 if endpoint
                 else [store.get_io_pointer(out) for out in outputs]
             )
@@ -224,16 +256,24 @@ def register(
             try:
                 repo = git.Repo(search_parent_directories=True)
                 component_run.set_git_hash(str(repo.head.object.hexsha))
-            except:
+            except Exception as e:
                 logging.info("No git repo found.")
 
+            # Add git tags
+            if get_git_tags() is not None:
+                component_run.set_git_tags(get_git_tags())
+
             # Add source code if less than 2^16
             func_source_code = inspect.getsource(func)
             if len(func_source_code) < 2 ** 16:
-                component_run.set_code_snapshot(bytes(func_source_code, "ascii"))
+                component_run.set_code_snapshot(
+                    bytes(func_source_code, "ascii")
+                )
 
             # Commit component run object to the DB
-            store.commit_component_run(component_run)
+            store.commit_component_run(
+                component_run, staleness_threshold=staleness_threshold
+            )
 
             return value
 
@@ -247,13 +287,47 @@ def get_git_hash() -> str:
     try:
         repo = git.Repo(search_parent_directories=True)
         return str(repo.head.object.hexsha)
-    except:
+    except Exception as e:
         logging.info("No git repo found.")
 
     return None
 
 
-# ------------------------- Basic retrieval functions ------------------------ #
+def get_git_tags() -> str:
+    """
+    Gets tags associated with commit of parent git repo, if exists
+    ref:https://stackoverflow.com/questions/34932306/get-tags-of-a-commit
+    """
+    try:
+        tagmap = {}
+        repo = git.Repo(search_parent_directories=True)
+        for t in repo.tags:
+            tagmap.setdefault(repo.commit(t), []).append(t)
+        tags = tagmap[repo.commit(repo.head.object.hexsha)]
+        return [tag.name for tag in tags]
+    except Exception as e:
+        logging.info("No git tag found")
+
+
+def add_notes_to_component_run(component_run_id: str, notes: str) -> str:
+    """Adds notes to component run."""
+    store = Store(_db_uri)
+    return store.add_notes_to_component_run(component_run_id, notes)
+
+
+def flag_output_id(output_id: str) -> bool:
+    """Sets the flag property of an IOPointer to true."""
+    store = Store(_db_uri)
+    return store.set_io_pointer_flag(output_id, True)
+
+
+def unflag_output_id(output_id: str) -> bool:
+    """Sets the flag property of an IOPointer to false."""
+    store = Store(_db_uri)
+    return store.set_io_pointer_flag(output_id, False)
+
+
+# ----------------- Basic retrieval functions ------------------- #
 
 
 def get_history(
@@ -278,7 +352,8 @@ def get_history(
     component_runs = []
     for cr in history:
         inputs = [
-            IOPointer.from_dictionary(iop.__dict__).to_dictionary() for iop in cr.inputs
+            IOPointer.from_dictionary(iop.__dict__).to_dictionary()
+            for iop in cr.inputs
         ]
         outputs = [
             IOPointer.from_dictionary(iop.__dict__).to_dictionary()
@@ -286,7 +361,13 @@ def get_history(
         ]
         dependencies = [dep.component_name for dep in cr.dependencies]
         d = copy.deepcopy(cr.__dict__)
-        d.update({"inputs": inputs, "outputs": outputs, "dependencies": dependencies})
+        d.update(
+            {
+                "inputs": inputs,
+                "outputs": outputs,
+                "dependencies": dependencies,
+            }
+        )
         component_runs.append(ComponentRun.from_dictionary(d))
 
     return component_runs
@@ -328,16 +409,20 @@ def get_component_run_information(component_run_id: str) -> ComponentRun:
     if not cr:
         raise RuntimeError(f"Component run with id {id} not found.")
     inputs = [
-        IOPointer.from_dictionary(iop.__dict__).to_dictionary() for iop in cr.inputs
+        IOPointer.from_dictionary(iop.__dict__).to_dictionary()
+        for iop in cr.inputs
     ]
     outputs = [
-        IOPointer.from_dictionary(iop.__dict__).to_dictionary() for iop in cr.outputs
+        IOPointer.from_dictionary(iop.__dict__).to_dictionary()
+        for iop in cr.outputs
     ]
     dependencies = [dep.component_name for dep in cr.dependencies]
     d = copy.deepcopy(cr.__dict__)
     if cr.code_snapshot:
         d.update({"code_snapshot": str(cr.code_snapshot.decode("utf-8"))})
-    d.update({"inputs": inputs, "outputs": outputs, "dependencies": dependencies})
+    d.update(
+        {"inputs": inputs, "outputs": outputs, "dependencies": dependencies}
+    )
     return ComponentRun.from_dictionary(d)
 
 
@@ -357,10 +442,10 @@ def get_components_with_tag(tag: str) -> typing.List[Component]:
     return components
 
 
-def get_recent_run_ids(limit: int = 50):
+def get_recent_run_ids(limit: int = 5, last_run_id=None):
     """Returns most recent component run ids."""
     store = Store(_db_uri)
-    return store.get_recent_run_ids(limit)
+    return store.get_recent_run_ids(limit, last_run_id)
 
 
 def get_io_pointer(io_pointer_id: str, create=True):
@@ -370,7 +455,7 @@ def get_io_pointer(io_pointer_id: str, create=True):
     return IOPointer.from_dictionary(iop.__dict__)
 
 
-# ------------------------ Complex retrieval functions ----------------------- #
+# --------------- Complex retrieval functions ------------------ #
 
 
 def backtrace(output_pointer: str):
@@ -384,10 +469,18 @@ def backtrace(output_pointer: str):
     component_runs = []
     for depth, cr in trace:
         inputs = [IOPointer.from_dictionary(iop.__dict__) for iop in cr.inputs]
-        outputs = [IOPointer.from_dictionary(iop.__dict__) for iop in cr.outputs]
+        outputs = [
+            IOPointer.from_dictionary(iop.__dict__) for iop in cr.outputs
+        ]
         dependencies = [dep.component_name for dep in cr.dependencies]
         d = copy.deepcopy(cr.__dict__)
-        d.update({"inputs": inputs, "outputs": outputs, "dependencies": dependencies})
+        d.update(
+            {
+                "inputs": inputs,
+                "outputs": outputs,
+                "dependencies": dependencies,
+            }
+        )
         component_runs.append((depth, ComponentRun.from_dictionary(d)))
 
     return component_runs
@@ -396,3 +489,12 @@ def backtrace(output_pointer: str):
 def web_trace(output_id: str):
     store = Store(_db_uri)
     return store.web_trace(output_id)
+
+
+def review_flagged_outputs():
+    """Finds common ComponentRuns for a group of flagged outputs.
+    Returns a list of ComponentRuns and occurrence counts in the
+    group of flagged outputs, sorted by descending count and then
+    alphabetically."""
+    store = Store(_db_uri)
+    return store.review_flagged_outputs()
