@@ -1,5 +1,7 @@
 from __future__ import annotations
 from datetime import datetime
+
+from sqlalchemy.sql.sqltypes import Boolean
 from mltrace.db.base import Base
 from sqlalchemy import (
     Column,
@@ -40,10 +42,16 @@ class Component(Base):
     description = Column(String)
     owner = Column(String)
     component_runs = relationship("ComponentRun", cascade="all, delete-orphan")
-    tags = relationship("Tag", secondary=component_tag_association, cascade="all")
+    tags = relationship(
+        "Tag", secondary=component_tag_association, cascade="all"
+    )
 
     def __init__(
-        self, name: str, description: str, owner: str, tags: typing.List[Tag] = []
+        self,
+        name: str,
+        description: str,
+        owner: str,
+        tags: typing.List[Tag] = [],
     ):
         self.name = name
         self.description = description
@@ -68,16 +76,26 @@ class IOPointer(Base):
 
     name = Column(String, primary_key=True)
     pointer_type = Column(Enum(PointerTypeEnum))
+    flag = Column(Boolean, default=False)
 
     def __init__(
-        self, name: str, pointer_type: PointerTypeEnum = PointerTypeEnum.UNKNOWN
+        self,
+        name: str,
+        pointer_type: PointerTypeEnum = PointerTypeEnum.UNKNOWN,
     ):
         self.name = name
         self.pointer_type = pointer_type
+        self.flag = False
 
     def set_pointer_type(self, pointer_type: PointerTypeEnum):
         self.pointer_type = pointer_type
 
+    def set_flag(self):
+        self.flag = True
+
+    def clear_flag(self):
+        self.flag = False
+
 
 component_run_input_association = Table(
     "component_runs_inputs",
@@ -97,7 +115,10 @@ component_run_dependencies = Table(
     "component_run_dependencies",
     Base.metadata,
     Column(
-        "component_run_id", Integer, ForeignKey("component_runs.id"), primary_key=True
+        "component_run_id",
+        Integer,
+        ForeignKey("component_runs.id"),
+        primary_key=True,
     ),
     Column(
         "depends_on_component_run_id",
@@ -113,7 +134,9 @@ class ComponentRun(Base):
 
     id = Column(Integer, primary_key=True)
     component_name = Column(String, ForeignKey("components.name"))
+    notes = Column(String)
     git_hash = Column(String)
+    git_tags = Column(PickleType)
     code_snapshot = Column(LargeBinary)
     start_timestamp = Column(DateTime)
     end_timestamp = Column(DateTime)
@@ -127,7 +150,8 @@ class ComponentRun(Base):
         "ComponentRun",
         secondary=component_run_dependencies,
         primaryjoin=id == component_run_dependencies.c.component_run_id,
-        secondaryjoin=id == component_run_dependencies.c.depends_on_component_run_id,
+        secondaryjoin=id
+        == component_run_dependencies.c.depends_on_component_run_id,
         backref="left_component_run_ids",
         cascade="all",
     )
@@ -136,13 +160,15 @@ class ComponentRun(Base):
     def __init__(self, component_name):
         """Initialize ComponentRun, or an instance of a Component's 'run.'"""
         self.component_name = component_name
+        self.notes = ""
         self.inputs = []
         self.outputs = []
         self.dependencies = []
         self.stale = []
 
     def set_start_timestamp(self, ts: datetime = None):
-        """Call this function to set the start timestamp to a specific timestamp or now."""
+        """Call this function to set the start timestamp
+        to a specific timestamp or now."""
         if ts is None:
             ts = datetime.utcnow()
 
@@ -152,7 +178,8 @@ class ComponentRun(Base):
         self.start_timestamp = ts
 
     def set_end_timestamp(self, ts: datetime = None):
-        """Call this function to set the end timestamp to a specific timestamp or now."""
+        """Call this function to set the end timestamp
+        to a specific timestamp or now."""
         if ts is None:
             ts = datetime.utcnow()
 
@@ -165,10 +192,20 @@ class ComponentRun(Base):
         """Code snapshot setter."""
         self.code_snapshot = code_snapshot
 
+    def add_notes(self, notes: str):
+        """Add notes describing details of component run"""
+        if not isinstance(notes, str):
+            raise TypeError("notes field must be of type str")
+        self.notes = notes
+
     def set_git_hash(self, git_hash: str):
         """Git hash setter."""
         self.git_hash = git_hash
 
+    def set_git_tags(self, git_tags: typing.List[str]):
+        """Git tag setter."""
+        self.git_tags = git_tags
+
     def add_staleness_message(self, message: str):
         """Staleness indicator."""
         self.stale = self.stale + [message]
@@ -178,7 +215,8 @@ class ComponentRun(Base):
         self._add_io(input, True)
 
     def add_inputs(self, inputs: typing.List[IOPointer]):
-        """Add a list of inputs (each element should be an instance of IOPointer)."""
+        """Add a list of inputs (each element should be
+        an instance of IOPointer)."""
         self._add_io(inputs, True)
 
     def add_output(self, output: IOPointer):
@@ -186,11 +224,14 @@ class ComponentRun(Base):
         self._add_io(output, False)
 
     def add_outputs(self, outputs: typing.List[IOPointer]):
-        """Add a list of outputs (each element should be an instance of IOPointer)."""
+        """Add a list of outputs (each element should be an
+        instance of IOPointer)."""
         self._add_io(outputs, False)
 
     def _add_io(
-        self, elems: typing.Union[typing.List[IOPointer], IOPointer], input: bool
+        self,
+        elems: typing.Union[typing.List[IOPointer], IOPointer],
+        input: bool,
     ):
         """Helper function to add inputs or outputs."""
         # Elems can be a list or a single IOPointer. Set to a list.
@@ -201,12 +242,16 @@ class ComponentRun(Base):
             self.outputs = list(set(self.outputs + elems))
 
     def set_upstream(
-        self, dependencies: typing.Union[typing.List[ComponentRun], ComponentRun]
+        self,
+        dependencies: typing.Union[typing.List[ComponentRun], ComponentRun],
     ):
-        """Set dependencies for this ComponentRun. API similar to Airflow set_upstream."""
+        """Set dependencies for this ComponentRun. API similar
+        to Airflow set_upstream."""
         # Dependencies can be a list or a single ComponentRun. Set to a list.
         dependencies = (
-            [dependencies] if not isinstance(dependencies, list) else dependencies
+            [dependencies]
+            if not isinstance(dependencies, list)
+            else dependencies
         )
 
         self.dependencies += dependencies
@@ -229,9 +274,13 @@ class ComponentRun(Base):
 
         # Show warnings if there are no dependencies or I/O.
         if len(self.inputs) == 0:
-            status_dict["msg"] += f"{self.component_name} ComponentRun has no inputs. "
+            status_dict[
+                "msg"
+            ] += f"{self.component_name} ComponentRun has no inputs. "
         if len(self.outputs) == 0:
-            status_dict["msg"] += f"{self.component_name} ComponentRun has no outputs. "
+            status_dict[
+                "msg"
+            ] += f"{self.component_name} ComponentRun has no outputs. "
         if len(self.dependencies) == 0:
             status_dict[
                 "msg"
