@@ -9,7 +9,11 @@ class Component(Base):
     """Component abstraction."""
 
     def __init__(
-        self, name: str, description: str, owner: str, tags: typing.List[str] = []
+        self,
+        name: str,
+        description: str,
+        owner: str,
+        tags: typing.List[str] = [],
     ):
         """Components should have a name, description, and owner.
         Optionally they will have tags."""
