@@ -7,10 +7,14 @@ import pprint
 
 class IOPointer(Base):
     def __init__(
-        self, name: str, pointer_type: PointerTypeEnum = PointerTypeEnum.UNKNOWN
+        self,
+        name: str,
+        pointer_type: PointerTypeEnum = PointerTypeEnum.UNKNOWN,
+        flag: bool = False,
     ):
         self._name = name
         self._pointer_type = pointer_type
+        self._flag = flag
 
     @property
     def name(self) -> str:
@@ -20,6 +24,10 @@ class IOPointer(Base):
     def pointer_type(self) -> PointerTypeEnum:
         return self._pointer_type
 
+    @property
+    def flag(self) -> bool:
+        return self._flag
+
     def __repr__(self):
         params = self.to_dictionary()
         params["pointer_type"] = params["pointer_type"].value
