@@ -1,3 +1,4 @@
+from mltrace.client import flag_output_id, unflag_output_id
 from dateutil import parser
 from flask import Blueprint, Flask, request, Response
 from http import HTTPStatus
@@ -10,7 +11,10 @@ from mltrace import (
     web_trace,
     get_recent_run_ids,
     get_io_pointer,
-    set_db_uri,
+    add_notes_to_component_run,
+    flag_output_id,
+    unflag_output_id,
+    review_flagged_outputs,
 )
 
 import copy
@@ -52,7 +56,8 @@ def component_run():
         return serialize_component_run(component, component_run)
     except RuntimeError:
         return error(
-            f"Component run ID {component_run_id} not found", HTTPStatus.NOT_FOUND
+            f"Component run ID {component_run_id} not found",
+            HTTPStatus.NOT_FOUND,
         )
 
 
@@ -64,9 +69,13 @@ def io_pointer():
     io_pointer_id = request.args["id"]
     try:
         res = get_io_pointer(io_pointer_id, create=False)
-        return json.dumps(IOPointer.from_dictionary(res.__dict__).to_dictionary())
+        return json.dumps(
+            IOPointer.from_dictionary(res.__dict__).to_dictionary()
+        )
     except RuntimeError:
-        return error(f"IOPointer {io_pointer_id} not found", HTTPStatus.NOT_FOUND)
+        return error(
+            f"IOPointer {io_pointer_id} not found", HTTPStatus.NOT_FOUND
+        )
 
 
 @api.route("/tag", methods=["GET"])
@@ -110,7 +119,9 @@ def history():
         )
         return str(history)
     except RuntimeError:
-        return error(f"Component {component_name} has no runs", HTTPStatus.NOT_FOUND)
+        return error(
+            f"Component {component_name} has no runs", HTTPStatus.NOT_FOUND
+        )
 
 
 @api.route("/component", methods=["GET"])
@@ -123,16 +134,15 @@ def component():
         component = get_component_information(component_name)
         return str(component)
     except RuntimeError:
-        return error(f"Component {component_name} not found", HTTPStatus.NOT_FOUND)
+        return error(
+            f"Component {component_name} not found", HTTPStatus.NOT_FOUND
+        )
 
 
 @api.route("/recent", methods=["GET"])
 def recent():
-    component_run_ids = (
-        get_recent_run_ids(request.args["limit"])
-        if "limit" in request.args
-        else get_recent_run_ids()
-    )
+    kwargs = request.args
+    component_run_ids = get_recent_run_ids(**kwargs)
     return json.dumps(component_run_ids)
 
 
@@ -148,4 +158,65 @@ def trace():
         return error(f"Output {output_id} not found", HTTPStatus.NOT_FOUND)
 
 
+@api.route("/notes", methods=["GET", "POST"])
+def add_notes():
+    if "id" not in request.json:
+        return error(f"ComponentRun id not specified.", HTTPStatus.NOT_FOUND)
+    component_run_id = request.json["id"]
+    notes = request.json["notes"]
+    try:
+        res = add_notes_to_component_run(component_run_id, notes)
+        return json.dumps(res)
+    except RuntimeError:
+        return error(
+            f"ComponentRun {component_run_id} unable to set notes to {notes}",
+            HTTPStatus.NOT_FOUND,
+        )
+
+
+@api.route("/flag", methods=["POST"])
+def flag():
+    if "id" not in request.json:
+        return error(f"IOPointer id not specified.", HTTPStatus.NOT_FOUND)
+    iopointer_name = request.json["id"]
+    try:
+        res = flag_output_id(iopointer_name)
+        return json.dumps(res)
+    except RuntimeError:
+        return error(
+            f"ComponentRun {iopointer_name} unable to be flagged for review",
+            HTTPStatus.NOT_FOUND,
+        )
+
+
+@api.route("/unflag", methods=["POST"])
+def unflag():
+    if "id" not in request.json:
+        return error(f"IOPointer id not specified.", HTTPStatus.NOT_FOUND)
+    iopointer_name = request.json["id"]
+    try:
+        res = unflag_output_id(iopointer_name)
+        return json.dumps(res)
+    except RuntimeError:
+        return error(
+            f"ComponentRun {iopointer_name} unable to be unflagged",
+            HTTPStatus.NOT_FOUND,
+        )
+
+
+@api.route("/review", methods=["GET"])
+def review():
+    try:
+        flagged_output_ids, trace_nodes_counts = review_flagged_outputs()
+        cr_ids_counts = [
+            (node.id, count) for node, count in trace_nodes_counts
+        ]
+        return json.dumps([flagged_output_ids, cr_ids_counts])
+    except RuntimeError:
+        return error(
+            "Flagged outputs unable to be reviewed",
+            HTTPStatus.NOT_FOUND,
+        )
+
+
 app.register_blueprint(api, url_prefix="/api")
